/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <hdb/db.h>

static fp_stream mout(stdout),merr(stderr);

main(int argc,char* argv[]) try
{
  if(argc<2) {
    mout<<"Usage: "<<argv[0]<<" [-s] <path>\n";
    return -1;
  }
  mstring path; int silent;
  if(argc>2) { silent=atoi(argv[1]); path=argv[2]; }
  else { silent=0; path=argv[1]; }
  mstring map; if(map.load(path)<0) THROW("hdbfix: could not open "<<path);
  filemap::head_t head;
  mstring version=First(sref(head.ident,sizeof(head.ident)));
  memcpy(&head,map.data(),sizeof(head));
  mstring cver=First(sref(head.ident,sizeof(head.ident)));
  mstring answer;
  int save=0;
  if(version!=cver) {
    merr<<path<<": Not a "<<version<<" file\n";
    return -1;
  }
  else {
    int top=head.frame.get(),last=0;
    while(top<head.filetop.get()&&top<map.size()) {
      filemap::rec_t* rec=(filemap::rec_t*)(map.data()+top);
      if(rec->mem.get()<=0) break;
      if(silent) {
        if(silent==top) {
          mout<<top<<" *************\n";
          for(int i=0;i<65536;i++) {
            char c=map.data()[top+i];
            if(cset_printable.test(c)) mout.put(c);
          }
          mout<<"\n\n";
        }
      }
      else {
        mout<<top<<" *************\n";
        for(int i=0;i<1024;i++) {
          char c=map.data()[top+i];
          if(cset_printable.test(c)) mout.put(c);
        }
        mout<<"\n\n";
      }
      last=top; top+=rec->mem.get();
    }
    if(top==head.filetop.get()&&head.filesize.get()==map.size()) {
      if(!silent) mout<<path<<": OK\n";
    }
    else {
      mout<<path<<": CORRUPTED\n";
      if(silent) return 0;
      mout<<"\n";
      mout<<"Head info:\n";
      mout<<"Size:     "<<head.filesize.get()<<"\n";
      mout<<"Frame:    "<<head.frame.get()<<"\n";
      mout<<"Top:      "<<head.filetop.get()<<"\n";
      mout<<"\n";

      mout<<"Actual sizes:\n";
      mout<<"Size:     "<<map.size()<<"\n";
      mout<<"Last:     "<<last<<"\n";
      mout<<"End:      "<<top<<"\n";
      mout<<"\n";

      if(map.size()!=head.filesize.get()) {
        mout<<"Filesizes differ - do you want to set the filesize to "
            <<head.filesize.get()<<"?\n";
        getline(answer,stdin); answer.tolower();
        if(*answer=='y') {
          map.resize(head.filesize.get());
          save=1;
          mout<<"Done.\n";
        }
        else {
          mout<<"No change - quitting.\n";
          return 0;
        }
      }
      
      int nend=0,end=head.filetop.get();
      for(int p=head.filetop.get()-4;p>=last;p-=4){
        unsigned val=*(unsigned*)(map.data()+p);
        if(p+val==end) { nend++; end=p; }
      }
      int gap=end-top;

      mout<<"GAP DATA\n";
      mout<<"==============================\n";
      for(int i=top;i<end;i++) {
        if(cset_printable.test(map[i])) mout.put(map[i]);
        else if(map[i]) mout<<"["<<(int)map[i]<<"]";
      }
      mout<<"\n==============================\n";
      mout<<"\n";

      mout<<"At-end sequence:\n";
      mout<<"Start:   "<<end<<"\n";
      mout<<"Gap:     "<<gap<<"\n";
      mout<<"\n";

      if(gap>0) {
        mout<<"Do you want to mark the gap as deleted?\n";
        getline(answer,stdin); answer.tolower();
        if(*answer=='y') {
          filemap::rec_t* rec=(filemap::rec_t*)(map.data()+top);
          rec->mem.set(gap);
          rec->caps.set(0);
          save=1;
          mout<<"Done.\n";
        }
        else {
          mout<<"No change - quitting.\n";
          return 0;
        }
      }
    }
  }
  
  if(save) map.save(path);

  return 0;
}
catch(const merror_t& e) {
  merr<<e.desc<<"\n";
  return -1;
}
catch(...) {
  merr<<"FATAL ERROR\n";
  return -1;
}
