/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <mime/mailbox.h>

static int FromHead(const sref& body)
{
  sref tag,from,rest=body;
  Split(tag,rest);
  Split(from,rest);
  return tag==from_tag&&htcTime(0,rest)>=0;
}

/////////////////////////////////////////////////////////////////////

const mstring nextmsg_tag="\nFrom ";
const mstring sep_tag="--";

Message::Message()
{
}

int Message::Special(sref line)
{
  line=Rest(line);
  from=from0=First(line);
  date=Rest(line);
  idate=htcTime(0,date);
  return idate>=0;
}

int Message::Handle(const sref& name,
                    const sref& rawval,
                    const sref& val,
                    const rrpile& opts)
{
  mstring mime;
  if(Mime::Handle(name,rawval,val,opts)) return 1;
  else if(eqn(name,from_tag)) {
    swrite sw(from);
    decodeHeader(sw,mime,rawval);
  }
  else if(eqn(name,subject_tag)) {
    swrite sw(subject);
    decodeHeader(sw,mime,rawval);
  }
  else if(eqn(name,to_tag)) {
    swrite sw(to);
    decodeHeader(sw,mime,rawval);
  }
  else if(eqn(name,msgid_tag)) msgid=rawval;
  else if(eqn(name,received_tag)) received.push_back(rawval);
  else return 0;
  return 1;
}

void Message::putHead(mstream& out) const
{
  if(from0.nempty()) { out<<"From "<<from0<<" "<<date<<"\r\n"; }
  if(putval(out,from_tag,from)) putend(out);
  if(putval(out,subject_tag,subject)) putend(out);
  if(putval(out,to_tag,to)) putend(out);
  if(putval(out,msgid_tag,msgid)) putend(out);
  for(spile::const_iterator p=received.begin();p!=received.end();p++)
    if(putval(out,received_tag,*p)) putend(out);
  Mime::putHead(out);
}

///////////////////////////////////////////////////////////////

void AppendMessage(FILE* fp,const sref& msg)
{
  fwrite(msg.data(),1,msg.size(),fp);
}

void AppendMessages(FILE* fp,const Mailbox& box)
{
  for(int i=0;i<box.size();i++) AppendMessage(fp,box[i]);
}

int PutMessages(const Mailbox& box,const mstring& path)
{
  FILE* fp=fopen(path.c_str(),"w+");
  if(!fp) return 0;
  AppendMessages(fp,box);
  return fclose(fp)==0;
}

int AppendMessages(const Mailbox& box,const mstring& path)
{
  FILE* fp=fopen(path.c_str(),"a+");
  if(!fp) return 0;
  AppendMessages(fp,box);
  return fclose(fp)==0;
}

///////////////////////////////////////////////////////////////

int GetMessages(Mailbox& box,const mstring& path)
{
  mstring buffer; if(buffer.load(path)<0) return -1;
  sref next,rest=buffer;
  int old=box.size();
  while(rest.nempty()) {
    Message msg(rest);
    if(msg.from0.empty()) break; // should not happen, I guess

    if(msg.cont_length.nempty())
      next=msg.body.right(atoi(msg.cont_length));
    else if(msg.cont_type_boundary.nempty())
      next=msg.body.past_first(sep_tag+msg.cont_type_boundary+sep_tag).right_first(nextmsg_tag).popf();
    else
      next=rest.right_first(nextmsg_tag).popf();

    box.push_back(rest.left(next-rest));

    rest=next.right_first_not_of(cset_ws);
    while(rest.nempty()&&!FromHead(rest)) rest=rest.right_first(nextmsg_tag).popf();
  }
  return box.size()-old;
}
