/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <mime/pack.h>

const char hex2char[16]={
  '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
};
const unsigned char2hex[256]={
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,0,0,0,0,0,0,
  0,10,11,12,13,14,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,10,11,12,13,14,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
};

mstream& cashex(mstream& out,unsigned char c) {
  return out.put(hex2char[c>>4]).put(hex2char[c&15]);
}
char hexasc(unsigned char c1,unsigned char c2) {
  return ((char2hex[c1]<<4)|char2hex[c2]);
}

void Pack(mstream& out,const sref& s)
{
  static const charset escaped("&=%+\"'? ");
  static const charset conv=~cset_printable|escaped;
  sref q,p=s;
  while(p.nempty()) {
    p=p.adv(q=p.left_first_of(conv));
    out<<q;
    while(conv.test(p.front())) {
      char c=*p; p=p.popf();
      if(c==' ') out.put('+');
      else {
        out.put('%');
        cashex(out,c);
      }
    }
  }
}
  
void Pack(mstream& out,const rpile& list)
{
  for(rpile::const_iterator p=list.begin();p!=list.end();++p) {
    Pack(out,*p);
    out.put('&');
  }
}

void Pack(mstream& out,const rrpile& list)
{
  for(rrpile::const_iterator p=list.begin();p!=list.end();++p) {
    Pack(out,p->first);
    out.put('=');
    Pack(out,p->second);
    out.put('&');
  }
}

///////////////////////////////////////////////////////////////

void Unpack(mstream& out,const sref& in)
{
  static const charset tags("%+");
  sref e,p=in;
  while(p.nempty()) {
    p=p.adv(e=p.left_first_of(tags));
    out<<e;
    while(tags.test(p.front())) {
      char c=*p; p=p.popf();
      if(c=='+') out.put(' ');
      else if(c=='%') {
        unsigned char c1=p[0],c2=p[1]; p=p.adv(2);
        out.put(hexasc(c1,c2));
      }
      else out.put(c);
    }
  }
}
  
void Unpack(spile& list,const sref& in)
{
  sref e,p=in;
  while(p.nempty()) {
    p=p.adv(e=p.left_first('&')).popf();
    if(e.nempty()) {
      list.push_back(mstring());
      swrite sw(list.back());
      Unpack(sw,e);
    }
  }
}

void Unpack(sspile& list,const sref& in)
{
  sref e1,e2,p=in;
  while(p.nempty()) {
    p=p.adv(e1=p.left_first('=')).popf();
    if(e1.nempty()) {
      p=p.adv(e2=p.left_first('&')).popf();
      list.push_back(sstring());
      swrite sw1(list.back().first);
      Unpack(sw1,e1);
      swrite sw2(list.back().second);
      Unpack(sw2,e2);
    }
  }
}

///////////////////////////////////////////////////////////////

void CRLF2LF(mstream& out,const sref& in)
{
  sref first,rest=in;
  while(Splitln(first,rest)) out<<first<<"\n";
}

void LF2CRLF(mstream& out,const sref& in)
{
  sref first,rest=in;
  while(Splitln(first,rest)) out<<first<<CRLF;
}

