/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <mt/lock.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/utsname.h>
#include <signal.h>
using namespace std;

const mstring LOCK=".lock";

void LockFile(const mstring& path,int prot) throw(merror_t)
{
  mstring name=path+LOCK; int fd;
  while((fd=open(name.c_str(),O_CREAT|O_WRONLY|O_TRUNC|O_EXCL,prot))==-1&&
        errno==EEXIST) sleep(1);
  if(fd<0) THROW("lock: could not create lock file "<<name);
  close(fd);
}

int remove(const char* path);
void UnlockFile(const mstring& path)
{
  remove((path+LOCK).c_str());
}

///////////////////////////////////////////////////////////////////

proclock_t::proclock_t(const mstring& p,int prot) : pid(0),path(p+LOCK)
{
  mstring buf,name=path;
  int fd=open(path.c_str(),O_CREAT|O_WRONLY|O_TRUNC|O_EXCL,prot);
  if(fd<0) {
    if(errno==EEXIST) {
      buf.load(path);
      int oldpid=atoi(buf);
      if(kill(oldpid,0)==0) THROW("proclock: already running as "<<buf)
      else fd=open(path.c_str(),O_CREAT|O_WRONLY|O_TRUNC,prot);
    }
    else THROW("proclock: could not create lock file "<<path);
  }
  pid=getpid();
  buf.convert("%d",pid);
  write(fd,buf.data(),buf.size());
  close(fd);
}

proclock_t::~proclock_t()
{
  if(getpid()==pid) remove(path.c_str());
}

///////////////////////////////////////////////////////////////////

nodelock_t::nodelock_t(const mstring& p,int prot) : pid(0),path(p+LOCK)
{
  struct utsname uts; uname(&uts);
  mstring buf;
  int fd=open(path.c_str(),O_CREAT|O_WRONLY|O_TRUNC|O_EXCL,prot);
  if(fd<0) {
    if(errno==EEXIST) {
      if(buf.load(path)>=0) {
        if(buf.nempty()&&buf!=sref(uts.nodename))
          THROW("nodelock: already locked on "<<buf);
      }
      else {
        fd=open(path.c_str(),O_CREAT|O_WRONLY|O_TRUNC,prot);
        pid=getpid();
        buf=uts.nodename;
        write(fd,buf.data(),buf.size());
        close(fd);
      }
    }
    else THROW("nodelock: could not create lock file "<<path);
  }
  else {
    pid=getpid();
    buf=uts.nodename;
    write(fd,buf.data(),buf.size());
    close(fd);
  }
}

nodelock_t::~nodelock_t()
{
  if(getpid()==pid) remove(path.c_str());
}

///////////////////////////////////////////////////////////////////

void ProcTerminate(const mstring& p,int stime)
{
  mstring buf,path=p+LOCK;
  if(buf.load(path)>=0) while(!kill(atoi(buf),SIGTERM)) sleep(stime);
}

