/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef STACKQUEH
#define STACKQUEH

#include <mt/mstring.h>
#include <mt/mlock.h>

template <class T> class stackque {
public:
  stackque(int m) {
    put=get=n=0;
    base=new T[caps=m];
    memset(&mutex,0,sizeof(mutex_t));
  }
  ~stackque() { delete base; }
  
  int push(const T& a) throw(merror_t) {
    MLOCK(mutex);
    if(n>=caps) THROW("stackque: Full");
    base[put++]=a;
    if(put==caps) put=0;
    return ++n;
  }

  int pop(T& a) {
    MLOCK(mutex);
    if(n) {
      a=base[get++];
      if(get==caps) get=0;
      return n--;
    }
    else return 0;
  }

  int capacity() const { return caps; }
  int size() const { return n; }
  int empty() const { return n==0; }
  int full() const { return n==caps; }
  void clear() { put=get=n=0; }
private:
  int put,get,n,caps;
  T* base;
  mutex_t mutex;
};

#endif
