/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <net/isocket.h>
#include <errno.h>

isocket::isocket(const in_addr& ia,int port,int backlog):
  pipe_stream(-1),rd(-1)
{
  if(port) {
    fd=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
    if(fd<0) THROW("isocket: Could not create");
    reuse(1);
  }
  else fd=0;
  sa.sin_family=2;
  sa.sin_port=htons(port);
  sa.sin_addr=ia;
  if(port) {
    if(bind(fd,(sockaddr*)&sa,sizeof(sa))==-1) THROW("isocket: Could not bind");
  }
  if(listen(fd,backlog)==-1) THROW("isocket: Could not listen");
  rd.setfd(fd);
}

isocket::isocket(int f,const sockaddr_in& ia):
  pipe_stream(f),rd(f)
{
  sa=ia;
  setflag(O_NONBLOCK|O_NDELAY);
  nodelay(1);
}

isocket* isocket::accept() throw(merror_t)
{
  sockaddr_in tsa; socklen_t tsalen;
  memset(&tsa,0,tsalen=sizeof(tsa));
  int tfd=::accept(fd,(sockaddr*)&tsa,&tsalen);
  if(tfd>0) return new isocket(tfd,tsa);
  if(errno!=EAGAIN) THROW("isocket: could not accept");
  return 0;
}
