/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef THRSERVH
#define THRSERVH

#include <net/server.h>
#include <mt/lookup.h>
#include <mt/mlock.h>

////////////////////////////////////////////////////////
//
// MT - safe
// Note that mutexes aren't needed here, since control
// is flowing from main to child, with possible child
// override in local variables.

struct thrServ {
  pthread_t ptid;
  mutex_t pmutex;
  mtcond_t pcond;
  volatile int tid,keepalive,running,termok;
  time_t first,last;
  htcServ* serv;
  isocket* sock;
  htcd_user user;
  
  thrServ(htcServ& server);
  ~thrServ();

  void Log(const sref& msg);
  void reset(isocket* s,int keep);

  int vacant() const { return sock==0; }
  int shutdown() { keepalive=0; return sock==0; }
  void disconnect() { keepalive=0; running=0; }
  void term() {
    keepalive=0;
    running=0;
    termok=1;
    MCONTINUE(pcond);
  }

  void enter();
  void session();
  int handle();
};

////////////////////////////////////////////////////////

struct thrCron {
  pthread_t ptid;
  volatile int tid,running;
  htcServ* serv;
  htcd_user user;

  thrCron(htcServ& server);

  void Log(const sref& msg);
  void enter();
};

#endif
