/*************************************************************************
 *
 * HTCd - Copyright (C) 1998-2006 Henrik Rydberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <serv/htcd.h>
#include <hdb/buffer.h>
#include <mt/lock.h>

static fp_stream mout(stdout),merr(stderr);
const mstring left="{",right="}";

static int unbalanced(const sref& in)
{
  static const charset tag("{[(");
  if(in.empty()) return 1;
  sref s=in.right_first_of(tag);
  while(s.nempty()) {
    int p=-1;
    switch(*s) {
    case '{': p=s.skip(cset_braces); break;
    case '[': p=s.skip(cset_indices); break;
    case '(': p=s.skip(cset_para); break;
    }
    if(p<0) return 1;
    s=s.adv(p).right_first_of(tag);
  }
  return 0;
}

main(int argc,char* argv[]) try
{
  NODELOCK(USERMAP);
  if(argc<2) THROW("Usage: "<<argv[0]<<" <docroot>");
  mstring path=argv[1];
  if(path.back()=='/') path=path.popb();
  HTCd htcd(path);
  tokmap user(htcd.toks);
  Setup(user,htcd);
  htcd_user thr(user,path);
  tcpReq tcp; tcp.port=0;
  httpReq hreq; hreq.method="GET";
  htcd_req req(htcd,thr.toks,mout,tcp,hreq);
  Env env(htcd.ops,req,XSYSTEM);
  if(argc>2) {
    for(int i=3;i<argc;i++) env.xpush(left+sref(argv[i])+right);
    mstring buffer; LoadBuffer(buffer,argv[2]);
    if(*buffer=='#') buffer.erase(0,buffer.find('\n')+1);
    mstream m;
    env.eval(m,buffer,1);
  }
  else {
    mstring buffer,line;
    while(!feof(stdin)) {
      merr<<"htcd>";
      buffer.clear();
      while(!feof(stdin)&&unbalanced(buffer)) {
        getline(line,stdin);
        buffer.append(sref(line));
        buffer+='\n';
      }
      try {
        env.parse(mout,buffer);
        mout<<"\n";
      }
      catch(const merror_t& e) {
        merr<<e.desc<<"\n";
      }
    }
  }
  return 0;
}
catch(const merror_t& e) {
  merr<<"ERROR: ["<<e.desc<<"]\n";
  return -1;
}


